﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using VSLiveWinRTData.VSLiveService;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;

namespace VSLiveWinRTData
{
    public class LocalStorageViewModel : BaseClass
    {
        bool _loaded = false;
        int sessionID = 1;
        public LocalStorageViewModel()
        {
            DeleteCommand = new SimpleCommand<Session>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            LoadData();
        }

        private ObservableCollection<Session> _sessions = new ObservableCollection<Session>();

        public ObservableCollection<Session> Sessions
        {
            get { return _sessions; }
            set { _sessions = value; OnPropertyChanged("Sessions"); }
        }

        private async Task LoadDataFromFile()
        {
            var folder = Windows.Storage.ApplicationData.Current.LocalFolder;
            var files = await folder.GetFilesAsync();
            string data = string.Empty;
            bool exists = files.Any(one => one.Name == "data.txt");
            if (!exists)
            {
                var file = folder.CreateFileAsync("data.txt");
            }
            else
            {
                var inputStream = await files.First(one => one.Name == "data.txt").OpenAsync(FileAccessMode.Read);
                var readStream = inputStream.GetInputStreamAt(0);

                DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(ObservableCollection<Session>));
                Sessions = (ObservableCollection<Session>)serializer.ReadObject(readStream.AsStreamForRead());
                if (Sessions.Count > 0)
                {
                    sessionID = Sessions.Max(s => s.SessionID) + 1;
                }
            }
            _loaded = true;
            SaveCommand.RaiseCanExecuteChanged();
        }

        private async Task SaveDataToFile()
        {
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(ObservableCollection<Session>));
            var folder = Windows.Storage.ApplicationData.Current.LocalFolder;
            var file = await folder.GetFileAsync("data.txt");
            var stream = await file.OpenStreamForWriteAsync();
            var writer = new DataWriter(stream.AsOutputStream());
            serializer.WriteObject(stream, Sessions);
            await stream.FlushAsync();
            stream.Dispose();
        }


        public async Task LoadData()
        {
            IsBusy = true;
            await LoadDataFromFile();
            IsBusy = false;
        }


        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }


        private Session selectedSession;

        public Session SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<Session> DeleteCommand { get; set; }

        public async void OnDelete(Session parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    IsBusy = true;
                    Sessions.Remove(parameter);
                    await SaveDataToFile();
                    IsBusy = false;
                }
                else
                {
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public async void OnSave(object parameter)
        {
            if (SelectedSession != null)
            {
                IsBusy = true;
                await SaveDataToFile();
                IsBusy = false;
            }
        }
        public bool CanSave(object parameter)
        {
            return _loaded;
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public void OnAdd(object parameter)
        {
            var session = new Session() { When = DateTime.Now, SessionID = sessionID++ };
            Sessions.Add(session);
            SelectedSession = session;
        }

        private string ConvertSessionToJson()
        {
            string json;
            var serializer = new DataContractJsonSerializer(typeof(Session));
            using (var stream = new MemoryStream())
            {
                serializer.WriteObject(stream, SelectedSession);
                stream.Flush();
                json = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);
            }
            return json;
        }

        private Session ConvertJsonToSession(string json)
        {
            Session session;
            var serializer = new DataContractJsonSerializer(typeof(Session));

            using (var stream = new MemoryStream(Encoding.UTF8.GetBytes(json)))
            {
                session = serializer.ReadObject(stream) as Session;
            }
            return session;
        }

        private bool hasError;

        public bool HasError
        {
            get { return hasError; }
            set { hasError = value; OnPropertyChanged("HasError"); }
        }
    }
}
