﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using VSLiveWinRTData.VSLiveService;
using Windows.Storage.Streams;
using Windows.UI.Xaml;

namespace VSLiveWinRTData
{
    public class RestViewModel : BaseClass
    {
#if DEBUG
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveService.svc/rest/";
#else
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveService.svc/rest/";
#endif
        private HttpClient _client;

        public RestViewModel()
        {
            DeleteCommand = new SimpleCommand<Session>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            LoadData();
        }

        private ObservableCollection<Session> _sessions;

        public ObservableCollection<Session> Sessions
        {
            get { return _sessions; }
            set { _sessions = value; OnPropertyChanged("Sessions"); }
        }


        public async Task LoadData()
        {
            IsBusy = true;
            _client = new HttpClient();
            _client.MaxResponseContentBufferSize = int.MaxValue;
            var response = await _client.SendAsync(new HttpRequestMessage(HttpMethod.Get, new Uri(_serviceUri + "GetList")));

            var data = await response.Content.ReadAsStringAsync();

            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(ObservableCollection<Session>));
            using (var stream = new MemoryStream(Encoding.UTF8.GetBytes(data)))
            {
                Sessions = serializer.ReadObject(stream) as ObservableCollection<Session>;
            }
            IsBusy = false;
        }


        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }


        private Session selectedSession;

        public Session SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<Session> DeleteCommand { get; set; }

        public async void OnDelete(Session parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    IsBusy = true;
                    _client = new HttpClient();
                    _client.MaxResponseContentBufferSize = int.MaxValue;
                    var content = new StringContent(string.Empty);
                    content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue("application/json");
                    var response = await _client.PostAsync(new Uri(_serviceUri + "Delete?sessionId=" + parameter.SessionID.ToString()), content);

                    var data = await response.Content.ReadAsStringAsync();
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
                else
                {
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public async void OnSave(object parameter)
        {
            if (SelectedSession != null)
            {
                IsBusy = true;
                string method = "Update";
                if (selectedSession.SessionID == 0)
                {
                    method = "Create";
                }
                _client = new HttpClient();
                _client.MaxResponseContentBufferSize = int.MaxValue;
                var content = new StringContent(ConvertSessionToJson());
                content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue("application/json");
                var response = await _client.PostAsync(new Uri(_serviceUri + method), content);

                var data = await response.Content.ReadAsStringAsync();

                var session = ConvertJsonToSession(data);
                Sessions[Sessions.IndexOf(selectedSession)] = session;
                SelectedSession = session;
                IsBusy = false;
            }
        }
        public bool CanSave(object parameter)
        {
            return (_client != null);
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public void OnAdd(object parameter)
        {
            if (_client != null)
            {
                var session = new Session() { When = DateTime.Now };
                Sessions.Add(session);
                SelectedSession = session;
            }
        }

        private string ConvertSessionToJson()
        {
            string json;
            var serializer = new DataContractJsonSerializer(typeof(Session));
            using (var stream = new MemoryStream())
            {
                serializer.WriteObject(stream, SelectedSession);
                stream.Flush();
                json = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);
            }
            return json;
        }

        private Session ConvertJsonToSession(string json)
        {
            Session session;
            var serializer = new DataContractJsonSerializer(typeof(Session));

            using (var stream = new MemoryStream(Encoding.UTF8.GetBytes(json)))
            {
                session = serializer.ReadObject(stream) as Session;
            }
            return session;
        }

        private bool hasError;

        public bool HasError
        {
            get { return hasError; }
            set { hasError = value; OnPropertyChanged("HasError"); }
        }
    }
}
