﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using SQLite;
using VSLiveWinRTData.VSLiveService;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;

namespace VSLiveWinRTData
{
    public class SQLiteViewModel : BaseClass, IDisposable
    {
        bool _loaded = false;
        SQLiteConnection connection;
        public SQLiteViewModel()
        {
            DeleteCommand = new SimpleCommand<DbSession>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            LoadData();
        }

        private ObservableCollection<DbSession> _DbSessions = new ObservableCollection<DbSession>();

        public ObservableCollection<DbSession> Sessions
        {
            get { return _DbSessions; }
            set { _DbSessions = value; OnPropertyChanged("DbSessions"); }
        }

        private async Task LoadDataFromDb()
        {
            var folder = Windows.Storage.ApplicationData.Current.LocalFolder;
            var files = await folder.GetFilesAsync();
            string data = string.Empty;
            if (!files.Any(one => one.Name == "demo.sqlite"))
            {
                try
                {
                    connection = new SQLiteConnection(Path.Combine(folder.Path, "demo.sqlite"));
                }
                catch (Exception ex)
                {
                    var er = ex;
                    throw;
                }
                
                connection.CreateTable<DbSession>();

            }
            else
            {
                Sessions.Clear();
                try
                {
                    connection = new SQLiteConnection(Path.Combine(folder.Path, "demo.sqlite"));

                    foreach (var one in connection.Table<DbSession>().OrderBy(one => one.Title))
                    {
                        Sessions.Add(one);
                    }

                }
                catch (Exception ex)
                {
                    var er = ex;
                    throw;
                }
            }
            _loaded = true;
            SaveCommand.RaiseCanExecuteChanged();
        }


        public async Task LoadData()
        {
            IsBusy = true;
            await LoadDataFromDb();
            IsBusy = false;
        }


        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }


        private DbSession selectedSession;

        public DbSession SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<DbSession> DeleteCommand { get; set; }

        public async void OnDelete(DbSession parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    IsBusy = true;
                    connection.RunInTransaction(() =>
                        {
                            connection.Delete(parameter);
                        });
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
                else
                {
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public async void OnSave(object parameter)
        {
            if (SelectedSession != null)
            {
                IsBusy = true;
                if (selectedSession.SessionID > 0)
                {
                    connection.RunInTransaction(() =>
                    {
                        connection.Update(selectedSession);
                    });
                }
                else
                {
                    connection.RunInTransaction(() =>
                    {
                        connection.Insert(selectedSession);
                    });
                }
                IsBusy = false;
            }
        }
        public bool CanSave(object parameter)
        {
            return _loaded;
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public void OnAdd(object parameter)
        {
            var dbSession = new DbSession() { When = DateTime.Now };
            Sessions.Add(dbSession);
            SelectedSession = dbSession;
        }

        private bool hasError;

        public bool HasError
        {
            get { return hasError; }
            set { hasError = value; OnPropertyChanged("HasError"); }
        }

        public void Dispose()
        {
            if (connection != null)
            {
                connection.Dispose();
            }
        }
    }
}
