﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data.Services.Client;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using VSLiveWinRTData.VSLiveService;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;

namespace VSLiveWinRTData
{
    public class WcfDataServicesViewModel : BaseClass
    {
#if DEBUG
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveWcfDataService.svc/";
#else
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveWcfDataService.svc/";
#endif

        private readonly VSLiveWinRTData.ODataService.Context _oDataClient;
        CoreDispatcher _dispatcher;

        public WcfDataServicesViewModel()
        {
            DeleteCommand = new SimpleCommand<VSLiveWinRTData.ODataService.Session>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            _oDataClient = new VSLiveWinRTData.ODataService.Context(new Uri(_serviceUri));
        }


        private DataServiceCollection<VSLiveWinRTData.ODataService.Session> _sessions;

        public DataServiceCollection<VSLiveWinRTData.ODataService.Session> Sessions
        {
            get { return _sessions; }
            set { _sessions = value; OnPropertyChanged("Sessions"); }
        }


        async public void LoadData(CoreDispatcher dispatcher)
        {
            try
            {
                IsBusy = true;
                _dispatcher = dispatcher;

                var query = (DataServiceQuery<VSLiveWinRTData.ODataService.Session>)_oDataClient.Sessions.OrderBy(session => session.Title);
                var data = await query.ExecuteAsync();

                Sessions = new DataServiceCollection<VSLiveWinRTData.ODataService.Session>(_oDataClient);

                Sessions.Load(data);

                IsBusy = false;

            }
            catch (DataServiceClientException ex)
            {
                // handle error message
            }
        }


        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }


        private VSLiveWinRTData.ODataService.Session selectedSession;

        public VSLiveWinRTData.ODataService.Session SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<VSLiveWinRTData.ODataService.Session> DeleteCommand { get; set; }

        public void OnDelete(VSLiveWinRTData.ODataService.Session parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    IsBusy = true;

                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
                else
                {
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public void OnSave(object parameter)
        {
            IsBusy = true;
            _oDataClient.BeginSaveChanges(
             SaveChangesOptions.Batch,
             result =>
             {
                 _dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
                     if (result.IsCompleted)
                     {
                         var client = result.AsyncState as VSLiveWinRTData.ODataService.Context;
                         Debug.Assert(client != null, "client != null");
                         DataServiceResponse response = client.EndSaveChanges(result);
                         if (response.IsBatchResponse)
                         {
                             var errors = new StringBuilder();
                             foreach (ChangeOperationResponse change in response)
                             {
                                 if (change.Error != null)
                                 {
                                     errors.Append(change.Error.Message);
                                 }
                             }
                             if (errors.Length != 0)
                             {
                                 HasError = true;

                             }
                             else
                             {
                                 HasError = false;
                             }
                         }
                         IsBusy = false;
                     }
                 });
                
             }, _oDataClient);
        }

        public bool CanSave(object parameter)
        {
            return (_oDataClient != null);
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public void OnAdd(object parameter)
        {
            if (_oDataClient != null)
            {
                var session = new VSLiveWinRTData.ODataService.Session() { When = DateTime.Now };
                Sessions.Add(session);
                SelectedSession = session;
            }
        }

        private bool hasError;

        public bool HasError
        {
            get { return hasError; }
            set { hasError = value; OnPropertyChanged("HasError"); }
        }

      

    }
}
